<?php
/**
 *
 * @category        modules
 * @package         MiniSlides v1.0
 * @author          Dev4me / Ruud Eisinga
 * @link			https://dev4me.com/modules-snippets/opensource/minislides/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.8.x / WBCE
 * @requirements    PHP 7.2 and higher
 * @version         1.0.0
 * @lastmodified    Februari 10, 2022
 *
 */

 
//Used for some default values and internal functions
require_once (WB_PATH.'/framework/functions.php');

if (!function_exists('getMiniSlidesImages')) {
	function getMiniSlidesImages($basedir,$dir, $section_id,$sorting=null) {
		$retval = array();
		if(substr($dir, -1) != "/")  $dir .= "/";
		$fulldir = $basedir.''.$dir;
		$d = @dir($fulldir) or die("getImages: Failed opening directory $fulldir for reading");
		while(false !== ($entry = $d->read())) {
		  if($entry[0] == ".") continue;
		  if(isImageFile($fulldir.$entry)) {
		    $caption = minislides_get_caption($entry, $section_id);
			$retval[] = array( "file" => ''.$dir.$entry, "caption" => $caption );
		  }
		}
		$d->close();
		minislides_array_sort_by('file',$retval);
		if($sorting) {
			$sarray = explode('|',$sorting);
			$first = array();
			foreach($sarray as $element) {
				foreach($retval as $key => $value) {
					if($value['file'] == $dir.$element) {
						$first[] = $value;
						unset($retval[$key]);
						continue;
					}	 
				} 
			}
			$retval = array_merge($first,$retval);
		}
		return $retval;
	}
}

if (!function_exists('getMiniSlidesImageList')) {
	function getMiniSlidesImageList($basedir,$curdir,$baseurl,$sid) {
		global $database;
		$get_settings = $database->query("SELECT * FROM ".TABLE_PREFIX."mod_minislides WHERE section_id = '$sid'");
		$settings = $get_settings->fetchRow();
		$ratio = $settings['ratio'];
		$sorting = $settings['sorting'];
		$images = getMiniSlidesImages($basedir,$curdir,$sid,$sorting); 
		$imglist = '<div id="sortable-'.$sid.'">';
		if($images) {
			foreach($images as $img) { 
				if (!file_exists($basedir.$curdir.'thumbs/'.basename($img['file']))) {
					cropImage($basedir.$curdir.basename($img['file']), $basedir.$curdir.'thumbs/'.basename($img['file']), $ratio, 200);
				}
				$imglist .= '<span data-id="'.basename($img['file']).'" class="imgholder"><img class="mgthumb" src="'.$baseurl.$curdir.'thumbs/'.basename($img['file']).'?t='.time().'" alt="'.$baseurl.$curdir.basename($img['file']).'" title="'.$img['caption'].'">
				<img class="edit edit-'.$sid.'" data-caption="'.htmlentities($img['caption']).'" data-filename="'.basename($img['file']).'" src="'.WB_URL.'/modules/minislides/edit.png" />
				<img class="delete del-'.$sid.'" data-filename="'.basename($img['file']).'" src="'.WB_URL.'/modules/minislides/delete.png" />
				</span>'; 
			} 
		}
		$imglist .= "</div>
		<script>
			var el = document.getElementById('sortable-$sid');
			var sortable = Sortable.create(el,{
			 animation: 150,
			 onSort: function (e) {
					var items = e.to.children;
					var result = [];
					for (var i = 0; i < items.length; i++) {
						result.push($(items[i]).data('id'));
					}
					$.ajax({url:'".WB_URL."/modules/minislides/ajax.php',type: 'POST',data : {section_id:'$sid',function:'sort',arr : result}});
				}
			});
		</script>";
		return $imglist;	
	}
}

if (!function_exists('cropImage')) {
	function cropImage($image, $output_file, $crop = null, $size = null) {
		// ini_set('memory_limit', '512M');
		$image = ImageCreateFromString(file_get_contents($image));
echo "<br>Cropping $output_file > $crop<br>";

		//if (is_resource($image) === true) {
		if (is_resource($image) || $image instanceof \GdImage) {
			$x = 0;
			$y = 0;
			$width = imagesx($image);
			$height = imagesy($image);

			/*
			CROP (Aspect Ratio) Section
			*/

			if (is_null($crop) === true) {
				$crop = array($width, $height);
			} else {
				$crop = array_filter(explode(':', $crop));

				if (empty($crop) === true) {
						$crop = array($width, $height);
				} else {
					if ((empty($crop[0]) === true) || (is_numeric($crop[0]) === false)) {
							$crop[0] = $crop[1];
					} else if ((empty($crop[1]) === true) || (is_numeric($crop[1]) === false)) {
							$crop[1] = $crop[0];
					}
				}

				$ratio = array(0 => $width / $height, 1 => $crop[0] / $crop[1]);
				if ($ratio[0] > $ratio[1]) {
					$width = $height * $ratio[1];
					$x = (imagesx($image) - $width) / 2;
				}

				else if ($ratio[0] < $ratio[1]) {
					$height = $width / $ratio[1];
					$y = (imagesy($image) - $height) / 2;
				}

			}

			/*
			Resize Section
			*/

			if (is_null($size) === true) {
				$size = array($width, $height);
			}

			else {
				$size = array_filter(explode('x', $size));

				if (empty($size) === true) {
						$size = array(imagesx($image), imagesy($image));
				} else {
					if ((empty($size[0]) === true) || (is_numeric($size[0]) === false)) {
							$size[0] = round($size[1] * $width / $height);
					} else if ((empty($size[1]) === true) || (is_numeric($size[1]) === false)) {
							$size[1] = round($size[0] * $height / $width);
					}
				}
			}

echo "$size[0] * $size[1]<br>";
			$result = ImageCreateTrueColor($size[0], $size[1]);

			//if (is_resource($result) === true) {
			if (is_resource($result) || $result instanceof \GdImage) {
				ImageSaveAlpha($result, true);
				ImageAlphaBlending($result, true);
				ImageFill($result, 0, 0, ImageColorAllocate($result, 255, 255, 255));
				ImageCopyResampled($result, $image, 0, 0, $x, $y, $size[0], $size[1], $width, $height);

				ImageInterlace($result, true);
				imagejpeg($result, $output_file,90);
				imagedestroy($result);
			}
		}
		return false;
	 }
}


if (!function_exists('minislides_resize_image')) {
	function minislides_resize_image( $source, $output_file, $size, $height, $crop  ,$force_thumb = false) {
		ini_set('memory_limit', '512M');
		if(!$source) return;
		minislides_image_fix_orientation($source);
		$info = getimagesize($source);
		list($orig_w, $orig_h) = $info;
		$src_x = 0;
		$src_y = 0;

		if(!$crop) {

			if ($orig_w < $size && $orig_h < $height ) { 
				if(!$force_thumb) return false;  //Image smaller than resize size. Thumbs should still be created
				$crop_w = $size;
				$crop_h = $height;	
			}
			
			$crop_w = $size;
			$crop_h = $orig_h*($size/$orig_w);
			
			if($crop_h > $height) {
				$crop_w = $orig_w*($height/$orig_h);
				$crop_h = $height;			
			}

			$new_h = $crop_h;
			$new_w = $crop_w;
			
		} else {
			
			$w_ratio = ($size / $orig_w);
			$h_ratio = ($size / $orig_h);
			$new_h = $size;
			$new_w = $size;
				
			if ($orig_w < $size && $orig_h < $size ) { 
				if(!$force_thumb) return false;  //Image smaller than resize size. Thumbs should still be created
			}
			if ($orig_w > $orig_h ) {
				$crop_w = round($orig_w * $h_ratio);
				$crop_h = $size;
				$src_x = ceil( ( $orig_w - $orig_h ) / 2 );
				$src_y = 0;
			} elseif ($orig_w < $orig_h ) {
				$crop_h = round($orig_h * $w_ratio);
				$crop_w = $size;
				$src_x = 0;
				$src_y = ceil( ( $orig_h - $orig_w ) / 2 );
			} else {
				$crop_w = $size;
				$crop_h = $size;
				$src_x = 0;
				$src_y = 0;
			}
				
		}

		switch ( $info[2] ) {
		  case IMAGETYPE_GIF:
			$image = imagecreatefromgif($source);
		  break;
		  case IMAGETYPE_JPEG:
			$image = imagecreatefromjpeg($source);
		  break;
		  case IMAGETYPE_PNG:
			$image = imagecreatefrompng($source);
		  break;
		  default:
			return false;
		}

		$dest_img = imagecreatetruecolor($new_w,$new_h);
		   
		if ( ($info[2] == IMAGETYPE_GIF) || ($info[2] == IMAGETYPE_PNG) ) {
		  $trnprt_indx = imagecolortransparent($image);
		  if ($trnprt_indx >= 0) {
			$trnprt_color    = imagecolorsforindex($image, $trnprt_indx);
			$trnprt_indx    = imagecolorallocate($dest_img, $trnprt_color['red'], $trnprt_color['green'], $trnprt_color['blue']);
			imagefill($dest_img, 0, 0, $trnprt_indx);
			imagecolortransparent($dest_img, $trnprt_indx);
		  }
		  elseif ($info[2] == IMAGETYPE_PNG) {
			imagealphablending($dest_img, false);
			$color = imagecolorallocatealpha($dest_img, 0, 0, 0, 127);
			imagefill($dest_img, 0, 0, $color);
			imagesavealpha($dest_img, true);
		  }
		}

		imagecopyresampled($dest_img, $image, 0 , 0 , $src_x, $src_y, $crop_w, $crop_h, $orig_w, $orig_h); 
		imagedestroy($image);
		
		switch ( $info[2] ) {
		  case IMAGETYPE_GIF:
			imagegif($dest_img, $output_file);
		  break;
		  case IMAGETYPE_JPEG:
			imagejpeg($dest_img, $output_file,90);
		  break;
		  case IMAGETYPE_PNG:
			imagepng($dest_img, $output_file);
		  break;
		  default:
			return false;
		}
		imagedestroy($dest_img);
		return true;
	  }
}

if (!function_exists('isImageFile')) {
	function isImageFile($filename) {
		if(is_dir($filename)) return false;
		if(@!is_array(@getimagesize($filename))){
			return false;
		}
		return true;
	}
}

if (!function_exists('minislides_image_fix_orientation')) {
	function minislides_image_fix_orientation($filename) {
		if(extension_loaded("exif")) {
			$exif = @exif_read_data($filename);
			if (!empty($exif['Orientation'])) {
				$image = imagecreatefromjpeg($filename);
				switch ($exif['Orientation']) {
					case 3:
						$image = imagerotate($image, 180, 0);
						break;

					case 6:
						$image = imagerotate($image, -90, 0);
						break;

					case 8:
						$image = imagerotate($image, 90, 0);
						break;
				}

				imagejpeg($image, $filename, 90);
				imagedestroy($image);
			}
		}
	}
}

if (!function_exists('minislides_save_upload')) {
	
	function minislides_save_upload ($fieldname, $resize = 0, $ratio = '4:3', $pathToFolder = null, $thumbFolder = null , $overwrite = false, &$message = '') {
		$message = '';
		if(isset($_FILES[$fieldname]['tmp_name']) AND $_FILES[$fieldname]['tmp_name'] != '') {
			$filename = $_FILES[$fieldname]['name'];

			$path_parts = pathinfo($filename);
			$fileext = strtolower($path_parts['extension']);

			$new_filename = $pathToFolder.$filename;
			$thumb_filename = $thumbFolder.$filename;
			
			// Make sure the image is a jpg or png file
			if(!isImageFile($_FILES[$fieldname]['tmp_name'])) {
				$message = "Error: ".$filename. ' is invalid. Only .jpg, .gif or .png is allowed!';
				return false;
			}

			if (file_exists($new_filename) && !$overwrite) {
				$message = "Error: ".$filename. ' already exists!';
				return false;
			} else {
				$message = "Saving: ".$filename. '';
				move_uploaded_file($_FILES[$fieldname]['tmp_name'], $new_filename);
				change_mode($new_filename);
				if (file_exists($new_filename)) {
					cropImage($new_filename, $new_filename, $ratio, $resize);
					cropImage($new_filename, $thumb_filename, $ratio, 200);
					return true;
				}
			}
		}
		return false;
	}
}
if (!function_exists('minislides_save_caption')) {
	function minislides_save_caption ($section_id, $filename, $caption ) {
		global $database;
		if($caption == '-') $caption = '';
		if($database->get_one("SELECT `image` FROM ".TABLE_PREFIX."mod_minislidesimg WHERE `image`='$filename' AND `section_id`='$section_id'")) {
		//update
			$query = "UPDATE ".TABLE_PREFIX."mod_minislidesimg SET `title`='$caption' WHERE `image`='$filename' AND `section_id`='$section_id'";
		} else { 
		// insert
			$query = "INSERT INTO ".TABLE_PREFIX."mod_minislidesimg SET 
				`section_id`='$section_id',
				`image`='$filename',
				`title`='$caption'";
		}
		//die($query);
		$database->query($query);
	}
}
if (!function_exists('minislides_save_sorting')) {
	function minislides_save_sorting ($section_id, $sorting ) {
		global $database;
		$query = "UPDATE ".TABLE_PREFIX."mod_minislides SET `sorting`='$sorting' WHERE `section_id`='$section_id'";
		$database->query($query);
	}
}

if (!function_exists('minislides_get_upload_limit')) {
	function minislides_get_upload_limit() {
		$max_upload = (int)(ini_get('upload_max_filesize'));
		$max_post = (int)(ini_get('post_max_size'));
		$memory_limit = (int)(ini_get('memory_limit'));
		$upload_mb = min($max_upload, $max_post, $memory_limit);
		return $upload_mb;
	}
}

if (!function_exists('minislides_array_sort_by')) {
	function minislides_array_sort_by($key, array &$array) {
		return usort($array, function($x, $y) use ($key) {
			//return strnatcasecmp($x[$key] ?? null, $y[$key] ?? null);
			return strnatcasecmp($x[$key], $y[$key]);
		});
	}
}


if (!function_exists('minislides_get_caption')) {
	function minislides_get_caption($filename, $section_id) {
		global $database;
		$filename = $database->escapeString($filename);
		//die("SELECT `title` FROM ".TABLE_PREFIX."mod_minislidesimg WHERE `section_id`='$section_id' AND `image`='$filename'");
		if($caption = $database->get_one("SELECT `title` FROM ".TABLE_PREFIX."mod_minislidesimg WHERE `section_id`='$section_id' AND `image`='$filename'")) {
			return $caption;
		}
		return '';
		
		$info = pathinfo($filename);
		$caption = basename($filename,'.'.$info['extension']);
		$caption = str_replace(array('.','#','-','_','^'),' ',$caption);
		$caption = str_replace(array('1','2','3','4','5','6','7','8','9','0'),'',$caption);
		$caption = ucwords($caption);
		return $caption;
	}
}

if (!function_exists('minislides_create_dir')) {
	function minislides_create_dir($section_id) {
		require(dirname(__FILE__).'/info.php');
		$basedir = WB_PATH.MEDIA_DIRECTORY.'/'.$image_path.'/';
		$pathToFolder = $basedir.$section_id.'/';
		$thumbFolder = $pathToFolder.'thumbs/';
		make_dir($basedir);
		make_dir($pathToFolder);
		make_dir($thumbFolder);
	}
}


